# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import struct
from fcntl import ioctl

import defines

class VideoCapability:
    # wraps struct video_capability
    struct_format = '32siiiiiii'
    def __init__(self, buffer, fd):
        list = struct.unpack(self.struct_format, buffer)
        list = [_scrub_nulls(x) for x in list]

        self.name      = list[0]
        self.type      = list[1] # flags from VID_TYPE_*
        self.channels  = list[2]
        self.audios    = list[3]
        self.maxwidth  = list[4]
        self.maxheight = list[5]
        self.minwidth  = list[6]
        self.minheight = list[7]

    def get_type(self):
        list = []
        types = _get_list_from_define_prefix('VID_TYPE_')
        for type in types:
            if self.is_set_type(type):
                list.append(type)
        return list
    
    def is_set_type(self, name):
        macro = 'VID_TYPE_' + name
        return _is_set_from_define(macro, self.type)


class VideoChannel:
    # wraps struct video_channel
    # python docs say h == short == 2 bytes so -> == __u16
    struct_format = 'i32siihh'

    def __init__(self, buffer, fd):
        self._tuners = None
        self._fd = fd

        list = struct.unpack(self.struct_format, buffer)
        list = [_scrub_nulls(x) for x in list]

        self.channel   = list[0]
        self.name      = list[1]
        self.tuners    = list[2]
        self.flags     = list[3] # VIDEO_VC_*
        self.type      = list[4] # VIDEO_TYPE_*
        self.norm      = list[5]

    def get_type(self):
        return _string_from_define_prefix('VIDEO_TYPE_', self.type)

    def get_tuners(self):
        # returns the tuners for this channel
        if self._tuners:
            return self._tuners
        tuners = []

        # set the channel
        for i in range(0, self.tuners):
            buffer = struct.pack('i', i)
            tuner = object_from_ioctl(VideoTuner, self._fd,
                defines.VIDIOCGTUNER, buffer=buffer)
            tuners.append(tuner)
        return tuners            

class VideoPicture:
    # wraps struct video_picture
    struct_format = '7h'
    
    def __init__(self, buffer, fd):
        list = struct.unpack(self.struct_format, buffer)

        self.brightness = list[0]
        self.hue        = list[1]
        self.colour     = list[2]
        self.contrast   = list[3]
        self.whiteness  = list[4]
        self.depth      = list[5]
        self.palette    = list[6]

    def get_palette(self):
        # get a string representation of self.palette
        # which will be the define name, without VIDEO_PALETTE_
        return _string_from_define_prefix('VIDEO_PALETTE_', self.palette)

class VideoTuner:
    # wraps struct video_picture
    struct_format = 'i32sllihh'
    
    def __init__(self, buffer, fd):
        list = struct.unpack(self.struct_format, buffer)
        list = [_scrub_nulls(x) for x in list]

        self.tuner      = list[0]
        self.name       = list[1]
        self.rangelow   = list[2]
        self.rangehigh  = list[3]
        self.flags      = list[4]
        self.mode       = list[5]
        self.signal     = list[6]

    def get_mode(self):
        return _string_from_define_prefix('VIDEO_MODE_', self.mode)

    def get_flags(self):
        list = []
        flags = _get_list_from_define_prefix('VIDEO_TUNER_')
        for flag in flags:
            if self.is_set_flags(flag):
                list.append(flag)
        return list
    
    def is_set_flags(self, name):
        macro = 'VIDEO_TUNER_' + name
        return _is_set_from_define(macro, self.flags)

def _scrub_nulls(pyobject):
    # scrub all terminating nulls from pyobject if it is a string
    if not isinstance(pyobject, str):
        return pyobject

    pos = pyobject.find('\x00')
    if pos != -1:
        return pyobject[:pos]

    return pyobject

def object_from_ioctl(klazz, fd,  ioctlarg, buffer=None):
    """
    Create an object representing a structure from the ioctl
    Optionally you can specify a buffer of arguments to use for the ioctl,
    which will then be expanded
    """
    format = klazz.struct_format
    size = struct.calcsize(format)
    if not buffer:
        buffer = 'x' * size
    if not len(buffer) == size:
        buffer = buffer + 'x' * (size - len(buffer))

    buffer = ioctl(fd, ioctlarg, buffer)
    object = klazz(buffer, fd=fd)
    return object

def _string_from_define_prefix(prefix, value):
    # given the #define prefix, and the numeric value, return the suffix
    # of the macro matching the value
    for name in dir(defines):
        if name.startswith(prefix):
            if value == getattr(defines, name):
                return name[len(prefix):]

    return None

def _is_set_from_define(macro, flags):
    # given the full macro name, check if
    # that flag is set in the given flags
    if not hasattr(defines, macro):
        raise Exception, "No macro named %s" % macro
    value = getattr(defines, macro)    
    return value & flags
    
def _get_list_from_define_prefix(prefix):
    # returns the list of suffixes for this prefix
    list = []
    for name in dir(defines):
        if name.startswith(prefix):
            list.append(name[len(prefix):])
    return list
